@@ -16,6 +16,12 @@ setClass("H5DSetDescriptor",
 
 H5DSetDescriptor <- function(filepath, name, as.integer=FALSE)
 {
+    if (!is(filepath, "H5File")) {
+        filepath <- H5File(filepath, .no_rhdf5_h5id=TRUE)
+        on.exit(close(filepath))
+    }
+    name <- normarg_h5_name(name)
+
     xp <- .Call2("C_new_H5DSetDescriptor_xp", filepath, name, as.integer,
                                               PACKAGE="HDF5Array")
     reg.finalizer(xp, .destroy_H5DSetDescriptor_xp, onexit=TRUE)
@@ -32,11 +38,3 @@ setMethod("show", "H5DSetDescriptor",
         .Call2("C_show_H5DSetDescriptor_xp", object@xp, PACKAGE="HDF5Array")
 )
 
-### The R type returned by h5mread() is determined by arguments 'filepath',
-### 'name', and 'as.integer'.
-get_h5mread_returned_type <- function(filepath, name, as.integer=FALSE)
-{
-    .Call2("C_get_h5mread_returned_type", filepath, name, as.integer,
-           PACKAGE="HDF5Array")
-}
-
