@@ -4,7 +4,7 @@
 
 
 setClass("H5SparseMatrixSeed",
-    contains="Array",
+    contains=c("Array", "OutOfMemoryObject"),
     representation(
         "VIRTUAL",
 
@@ -61,7 +61,8 @@ setClass("CSR_H5SparseMatrixSeed", contains="H5SparseMatrixSeed")
 t.CSC_H5SparseMatrixSeed <- function(x)
 {
     x@dim <- rev(x@dim)
-    class(x) <- "CSR_H5SparseMatrixSeed"
+    x@dimnames <- rev(x@dimnames)
+    class(x) <- class(new("CSR_H5SparseMatrixSeed"))
     x
 }
 setMethod("t", "CSC_H5SparseMatrixSeed", t.CSC_H5SparseMatrixSeed)
@@ -70,7 +71,8 @@ setMethod("t", "CSC_H5SparseMatrixSeed", t.CSC_H5SparseMatrixSeed)
 t.CSR_H5SparseMatrixSeed <- function(x)
 {
     x@dim <- rev(x@dim)
-    class(x) <- "CSC_H5SparseMatrixSeed"
+    x@dimnames <- rev(x@dimnames)
+    class(x) <- class(new("CSC_H5SparseMatrixSeed"))
     x
 }
 setMethod("t", "CSR_H5SparseMatrixSeed", t.CSR_H5SparseMatrixSeed)
@@ -112,6 +114,33 @@ setMethod("dimnames", "H5SparseMatrixSeed",
 )
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### chunkdim() getter
+###
+
+### Does NOT access the file.
+setMethod("chunkdim", "CSC_H5SparseMatrixSeed",
+    function(x) c(nrow(x), min(ncol(x), 1L))
+)
+
+setMethod("chunkdim", "CSR_H5SparseMatrixSeed",
+    function(x) c(min(nrow(x), 1L), ncol(x))
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### is_sparse() and nzcount() methods
+###
+
+### This is about **structural** sparsity, not about quantitative sparsity
+### measured by sparsity().
+setMethod("is_sparse", "H5SparseMatrixSeed", function(x) TRUE)
+
+setMethod("nzcount", "H5SparseMatrixSeed",
+    function(x) h5length(x@filepath, .get_data_name(x@subdata, x@group))
+)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Low-level internal h5sparse data readers
 ###
@@ -125,8 +154,8 @@ read_h5sparse_component <- function(filepath, group, name,
         start <- list(start)
     if (!is.null(count))
         count <- list(count)
-    as.vector(h5mread(filepath, name, starts=start, counts=count,
-                                      as.integer=as.integer))
+    h5mread(filepath, name, starts=start, counts=count,
+            as.vector=TRUE, as.integer=as.integer)
 }
 
 ### Returns a numeric vector (integer or double).
@@ -183,10 +212,10 @@ read_h5sparse_component <- function(filepath, group, name,
 }
 
 ### The row (or column) indices stored in HDF5 dataset "indices" are 0-based
-### but we return them 1-based.
+### and we return them as such.
 .read_h5sparse_indices <- function(filepath, group, start=NULL, count=NULL)
     read_h5sparse_component(filepath, group, "indices",
-                            start=start, count=count, as.integer=TRUE) + 1L
+                            start=start, count=count, as.integer=TRUE)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -312,30 +341,30 @@ H5SparseMatrixSeed <- function(filepath, group, subdata=NULL,
     dim <- .get_sparse_matrix_dim(filepath, group, dim=dim)
 
     ## Get sparse layout to use ("CSC" or "CSR").
+    ## Note that R has the notions of rows and columns flipped w.r.t.
+    ## HDF5 so:
+    ## - "compressed sparse row" at the HDF5 level translates
+    ##   into "compressed sparse column" at the R level,
+    ## - "compressed sparse column" at the HDF5 level translates
+    ##   into "compressed sparse row" at the R level.
     layout <- .get_sparse_matrix_layout(filepath, group,
                                         sparse.layout=sparse.layout)
     if (layout == "CSC") {
-        expected_indptr_length <- dim[[2L]] + 1L
-        ## Because R has the notions of rows and columns flipped w.r.t.
-        ## HDF5, "compressed sparse row" at the HDF5 level translates
-        ## into "compressed sparse column" at the R level.
+        expected_indptr_len <- dim[[2L]] + 1L
         ans_class <- "CSC_H5SparseMatrixSeed"
     } else {
-        expected_indptr_length <- dim[[1L]] + 1L
-        ## Because R has the notions of rows and columns flipped w.r.t.
-        ## HDF5, "compressed sparse column" at the HDF5 level translates
-        ## into "compressed sparse row" at the R level.
+        expected_indptr_len <- dim[[1L]] + 1L
         ans_class <- "CSR_H5SparseMatrixSeed"
     }
 
     ## Get 'indptr_ranges'.
-    data_len <- h5length(filepath, .get_data_name(subdata, group))
+    nzcount <- h5length(filepath, .get_data_name(subdata, group))
     indices_len <- h5length(filepath, paste0(group, "/indices"))
-    stopifnot(data_len == indices_len)
+    stopifnot(indices_len == nzcount)
     indptr <- .read_h5sparse_indptr(filepath, group)
-    stopifnot(length(indptr) == expected_indptr_length,
+    stopifnot(length(indptr) == expected_indptr_len,
               indptr[[1L]] == 0L,
-              indptr[[length(indptr)]] == data_len)
+              indptr[[length(indptr)]] == nzcount)
     indptr_ranges <- data.frame(start=indptr[-length(indptr)] + 1,
                                 width=as.integer(diff(indptr)))
 
@@ -344,9 +373,165 @@ H5SparseMatrixSeed <- function(filepath, group, subdata=NULL,
 }
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### .load_CSC_H5SparseMatrixSeed
+###
+### Loads CSC_H5SparseMatrixSeed object 'x' into memory as a SVT_SparseMatrix
+### object, or selected columns only.
+### This is the workhorse behind the extract_sparse_array(), extract_array(),
+### and read_block_as_sparse() methods for H5SparseMatrixSeed objects, as
+### well as behind coercion from CSC_H5SparseMatrixSeed to SVT_SparseMatrix.
+### Does NOT propagate the dimnames.
+###
+### Notes:
+### - SparseArray:::make_SVT_SparseMatrix_from_CSC() will fail if
+###   supplied 'data'/'row_indices' arguments are long vectors because R
+###   does not support passing long vectors to the .Call interface yet!
+###   So we use a block strategy where we load blocks of adjacent columns
+###   and convert them to SVT_SparseMatrix objects, then cbind() all the
+###   objects together. By default, blocks are made of 125 millions
+###   data/indices elements.
+### - Supports parallelization via the 'BPPARAM' argument. However some
+###   quick testing with 'BiocParallel::MulticoreParam(2)' on a powerful
+###   Linux server seemed to indicate that it's not worth it. Execution
+###   time remained about the same but memory footprint increased
+###   significantly!
+
+.load_CSC_H5SparseMatrixSeed <- function(x, j=NULL,
+                                         DATABLOCKLEN=125000000L,
+                                         BPPARAM=NULL)
+{
+    stopifnot(is(x, "CSC_H5SparseMatrixSeed"),
+              isSingleInteger(DATABLOCKLEN), DATABLOCKLEN >= 0L)
+    if (is.null(j)) {
+        ans_ncol <- ncol(x)
+        w <- x@indptr_ranges[ , "width"]
+    } else {
+        stopifnot(is.integer(j))
+        ans_ncol <- length(j)
+        if (ans_ncol != 0L)
+            stopifnot(isStrictlySorted(j),
+                      1L <= j[[1L]], j[[ans_ncol]] <= ncol(x))
+        w <- x@indptr_ranges[j , "width"]
+    }
+    ans_dim <- c(nrow(x), ans_ncol)
+    ## 'cumsum(as.double(w))' instead of 'cumsum(w)' to avoid integer overflow.
+    ans_indptr <- c(0, cumsum(as.double(w)))
+    ans_nzcount <- ans_indptr[[length(ans_indptr)]]
+
+    ## DATABLOCKLEN == 0L means no block processing.
+    if (DATABLOCKLEN == 0L || ans_nzcount <= DATABLOCKLEN) {
+        if (is.null(j)) {
+            start <- count <- NULL
+        } else {
+            start <- x@indptr_ranges[j, "start"]
+            count <- x@indptr_ranges[j, "width"]
+        }
+        ans_data <- .read_h5sparse_data(x@filepath, x@group, x@subdata,
+                                        start=start, count=count)
+        ans_row_indices <- .read_h5sparse_indices(x@filepath, x@group,
+                                        start=start, count=count)
+        ans <- SparseArray:::make_SVT_SparseMatrix_from_CSC(ans_dim,
+                                        ans_indptr, ans_data, ans_row_indices)
+        return(ans)
+    }
+
+    ## Compute 'nblock' (will always be >= 2).
+    nblock <- ans_nzcount %/% DATABLOCKLEN
+    if (ans_nzcount %% DATABLOCKLEN != 0L)
+        nblock <- nblock + 1L
+
+    ## Partition column indices in ranges (nb of ranges is guaranteed to be
+    ## >= 1 and <= 'min(nblock, ans_ncol)').
+    col_ranges <- breakInChunks(ans_ncol, nblock)
+    ## There will be zero-width ranges if and only if 'nblock' > 'ans_ncol'.
+    ## Drop them.
+    col_ranges <- col_ranges[width(col_ranges) != 0L]
+    s <- start(col_ranges)
+    e <- end(col_ranges)
+
+    ## Load ranges of columns into SVT_SparseMatrix objects.
+    objects <- S4Arrays:::bplapply2(seq_along(col_ranges),
+        function(b, x, j, s, e) {
+            k1 <- s[[b]]
+            k2 <- e[[b]]
+            jj <- if (is.null(j)) k1:k2 else j[k1:k2]
+            ## Set 'DATABLOCKLEN' to 0L to disable block processing.
+            .load_CSC_H5SparseMatrixSeed(x, jj, DATABLOCKLEN=0L)
+        },
+        x, j, s, e,
+        BPPARAM=BPPARAM
+    )
+
+    ## Combine all objects together.
+    do.call(cbind, objects)
+}
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### extract_sparse_array() and extract_array() methods
+###
+
+.extract_sparse_array_from_CSC_H5SparseMatrixSeed <- function(x, index)
+{
+    j <- index[[2L]]
+    if (!is.null(j)) {
+        if (!is.integer(j))
+            j <- as.integer(j)
+        sort_j <- !isStrictlySorted(j)
+        if (sort_j) {
+            j0 <- j
+            j <- unique(sort(j))
+        }
+    }
+    svt <- .load_CSC_H5SparseMatrixSeed(x, j=j)
+    index2 <- list(index[[1L]], NULL)
+    if (!is.null(j) && sort_j)
+        index2[[2L]] <- match(j0, j)
+    extract_sparse_array(svt, index2)
+}
+
+setMethod("extract_sparse_array", "CSC_H5SparseMatrixSeed",
+    function(x, index)
+        .extract_sparse_array_from_CSC_H5SparseMatrixSeed(x, index)
+)
+
+setMethod("extract_sparse_array", "CSR_H5SparseMatrixSeed",
+    function(x, index)
+        t(.extract_sparse_array_from_CSC_H5SparseMatrixSeed(t(x), rev(index)))
+)
+
+setMethod("extract_array", "H5SparseMatrixSeed",
+    function(x, index) as.array(extract_sparse_array(x, index))
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Show
+###
+
+setMethod("show", "H5SparseMatrixSeed",
+    function(object)
+    {
+        cat(S4Arrays:::array_as_one_line_summary(object), ":\n", sep="")
+        cat("# dirname: ", dirname(object), "\n", sep="")
+        cat("# basename: ", basename(object), "\n", sep="")
+        cat("# group: ", object@group, "\n", sep="")
+    }
+)
+
+
+### -------------------------------------------------------------------------
+### OLD STUFF. To be deprecated soon!
+### -------------------------------------------------------------------------
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### .get_data_indices_by_col()
 ###
+### Used by .load_sparse_data(), extractNonzeroDataByCol(), and
+### extractNonzeroDataByRow() defined below in this file.
+###
 
 ### base::sequence() does not properly handle a 'from' that is >
 ### .Machine$integer.max so we implement a variant that does. Note that
@@ -379,12 +564,15 @@ H5SparseMatrixSeed <- function(filepath, group, subdata=NULL,
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### .extract_data_from_adjacent_cols()
 ###
+### Used by .load_sparse_data(), extractNonzeroDataByCol(), and
+### extractNonzeroDataByRow() defined below in this file.
+###
 
 ### 'j1' and 'j2' must be 2 single integers representing a valid range of
 ### col indices.
-### If 'as.sparse=FALSE', return a NumericList or IntegerList object parallel
+### If 'as.sparse=FALSE', returns a NumericList or IntegerList object parallel
 ### to 'j1:j2' i.e. with one list element per col index in 'j1:j2'.
-### If 'as.sparse=TRUE', return a SparseArraySeed object.
+### If 'as.sparse=TRUE', returns a SparseArraySeed object.
 .extract_data_from_adjacent_cols <- function(x, j1, j2, as.sparse=FALSE)
 {
     j12 <- j1:j2
@@ -396,7 +584,7 @@ H5SparseMatrixSeed <- function(filepath, group, subdata=NULL,
     if (!as.sparse)
         return(relist(ans_nzdata, PartitioningByWidth(count_per_col)))
     row_indices <- .read_h5sparse_indices(x@filepath, x@group,
-                                          start=start, count=count)
+                                          start=start, count=count) + 1L
     col_indices <- rep.int(j12, count_per_col)
     ans_nzindex <- cbind(row_indices, col_indices, deparse.level=0L)
     SparseArraySeed(dim(x), ans_nzindex, ans_nzdata, check=FALSE)
@@ -406,12 +594,11 @@ H5SparseMatrixSeed <- function(filepath, group, subdata=NULL,
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### .load_sparse_data()
 ###
-### This internal generic is the workhorse behind the extract_array(),
-### OLD_extract_sparse_array(), and read_sparse_block() methods for
-### H5SparseMatrixSeed objects.
+### Used by the OLD_extract_sparse_array() and read_sparse_block() methods
+### for H5SparseMatrixSeed objects defined below in this file.
 ###
 
-### Load sparse data using the "random" method.
+### Loads sparse data using the "random" method.
 ### This method is based on h5mread( , starts=list(start)) which retrieves
 ### an arbitrary/random subset of the data.
 ### 'i' must be NULL or an integer vector containing valid row indices.
@@ -419,13 +606,13 @@ H5SparseMatrixSeed <- function(filepath, group, subdata=NULL,
 ### be NULL.
 ### Both 'i' and 'j' can contain duplicates. Duplicates in 'i' have no effect
 ### on the output but duplicates in 'j' will produce duplicates in the output.
-### Return a SparseArraySeed object.
+### Returns a SparseArraySeed object.
 .load_random_csc_sparse_data <- function(x, i, j)
 {
     stopifnot(is.null(i) || is.numeric(i), is.numeric(j))
     data_indices <- .get_data_indices_by_col(x, j)
     idx2 <- unlist(data_indices, use.names=FALSE)
-    row_indices <- .read_h5sparse_indices(x@filepath, x@group, start=idx2)
+    row_indices <- .read_h5sparse_indices(x@filepath, x@group, start=idx2) + 1L
     col_indices <- rep.int(j, lengths(data_indices))
     if (!is.null(i)) {
         keep_idx <- which(row_indices %in% i)
@@ -439,13 +626,13 @@ H5SparseMatrixSeed <- function(filepath, group, subdata=NULL,
     SparseArraySeed(dim(x), ans_nzindex, ans_nzdata, check=FALSE)
 }
 
-### Load sparse data using the "linear" method.
+### Loads sparse data using the "linear" method.
 ### This method is based on h5mread( , starts=list(start), counts=list(count))
 ### which retrieves a linear subset of the data and should be more efficient
 ### than doing h5mread( , starts=list(seq(start, length.out=count))).
 ### 'j' must be NULL or a non-empty integer vector containing valid
 ### col indices. The output is not affected by duplicates in 'j'.
-### Return a SparseArraySeed object.
+### Returns a SparseArraySeed object.
 .load_linear_csc_sparse_data <- function(x, j)
 {
     if (is.null(j)) {
@@ -484,7 +671,7 @@ H5SparseMatrixSeed <- function(filepath, group, subdata=NULL,
 ### Duplicates in 'index[[1]]' are ok and won't affect the output.
 ### Duplicates in 'index[[2]]' are ok but might introduce duplicates
 ### in the output so should be avoided.
-### Return a SparseArraySeed object.
+### Returns a SparseArraySeed object.
 .load_csc_sparse_data <- function(x, index, method)
 {
     i <- index[[1L]]
@@ -498,7 +685,7 @@ H5SparseMatrixSeed <- function(filepath, group, subdata=NULL,
     ans
 }
 
-### Return a SparseArraySeed object.
+### Returns a SparseArraySeed object.
 setGeneric(".load_sparse_data", signature="x",
     function(x, index, method=c("auto", "random", "linear"))
         standardGeneric(".load_sparse_data")
@@ -522,57 +709,11 @@ setMethod(".load_sparse_data", "CSR_H5SparseMatrixSeed",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### extract_array()
+### OLD_extract_sparse_array() and read_sparse_block() methods
 ###
-
-.extract_array_from_H5SparseMatrixSeed <- function(x, index)
-{
-    ans_dim <- S4Arrays:::get_Nindex_lengths(index, dim(x))
-    if (any(ans_dim == 0L)) {
-        ## Return an empty matrix.
-        data0 <- .read_h5sparse_data(x@filepath, x@group, x@subdata,
-                                     start=integer(0))
-        return(array(data0, dim=ans_dim))
-    }
-    sas <- .load_sparse_data(x, index)  # I/O
-    extract_array(sas, index)  # in-memory
-}
-
-setMethod("extract_array", "H5SparseMatrixSeed",
-    .extract_array_from_H5SparseMatrixSeed
-)
-
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### chunkdim() getter
+### TODO: Deprecate this stuff.
 ###
 
-### Does NOT access the file.
-setMethod("chunkdim", "CSC_H5SparseMatrixSeed",
-    function(x) c(nrow(x), min(ncol(x), 1L))
-)
-
-setMethod("chunkdim", "CSR_H5SparseMatrixSeed",
-    function(x) c(min(nrow(x), 1L), ncol(x))
-)
-
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Taking advantage of sparsity
-###
-
-setMethod("sparsity", "H5SparseMatrixSeed",
-    function(x)
-    {
-        data_len <- h5length(x@filepath, .get_data_name(x@subdata, x@group))
-        1 - data_len / length(x)
-    }
-)
-
-### This is about **structural** sparsity, not about quantitative sparsity
-### measured by sparsity().
-setMethod("is_sparse", "H5SparseMatrixSeed", function(x) TRUE)
-
 .OLD_extract_sparse_array_from_H5SparseMatrixSeed <- function(x, index)
 {
     sas <- .load_sparse_data(x, index)  # I/O
@@ -614,8 +755,9 @@ setMethod("read_sparse_block", "H5SparseMatrixSeed",
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### extractNonzeroDataByCol() and extractNonzeroDataByRow()
 ###
-### Spelling: "nonzero" preferred over "non-zero". See:
-###   https://gcc.gnu.org/ml/gcc/2001-10/msg00610.html
+### TODO: Deprecate these 2 generics and their methods. These 2 generics are
+### weird and don't have good/strong use cases. I suspect nobody uses them
+### nor is aware of them.
 ###
 
 ### Extract nonzero data using the "random" method.
@@ -698,55 +840,3 @@ setMethod("extractNonzeroDataByRow", "CSR_H5SparseMatrixSeed",
     }
 )
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Coercion to dgCMatrix
-###
-
-.from_CSC_H5SparseMatrixSeed_to_dgCMatrix <- function(from)
-{
-    row_indices <- .read_h5sparse_indices(from@filepath, from@group)
-    indptr <- .read_h5sparse_indptr(from@filepath, from@group)
-    data <- .read_h5sparse_data(from@filepath, from@group, from@subdata)
-    sparseMatrix(i=row_indices, p=indptr, x=data, dims=dim(from),
-                 dimnames=dimnames(from))
-}
-
-setAs("CSC_H5SparseMatrixSeed", "dgCMatrix",
-    .from_CSC_H5SparseMatrixSeed_to_dgCMatrix
-)
-setAs("CSC_H5SparseMatrixSeed", "sparseMatrix",
-    .from_CSC_H5SparseMatrixSeed_to_dgCMatrix
-)
-
-.from_CSR_H5SparseMatrixSeed_to_dgCMatrix <- function(from)
-{
-    col_indices <- .read_h5sparse_indices(from@filepath, from@group)
-    indptr <- .read_h5sparse_indptr(from@filepath, from@group)
-    data <- .read_h5sparse_data(from@filepath, from@group, from@subdata)
-    sparseMatrix(j=col_indices, p=indptr, x=data, dims=dim(from),
-                 dimnames=dimnames(from))
-}
-
-setAs("CSR_H5SparseMatrixSeed", "dgCMatrix",
-    .from_CSR_H5SparseMatrixSeed_to_dgCMatrix
-)
-setAs("CSR_H5SparseMatrixSeed", "sparseMatrix",
-    .from_CSR_H5SparseMatrixSeed_to_dgCMatrix
-)
-
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Show
-###
-
-setMethod("show", "H5SparseMatrixSeed",
-    function(object)
-    {
-        cat(S4Arrays:::array_as_one_line_summary(object), ":\n", sep="")
-        cat("# dirname: ", dirname(object), "\n", sep="")
-        cat("# basename: ", basename(object), "\n", sep="")
-        cat("# group: ", object@group, "\n", sep="")
-    }
-)
-
