@@ -46,7 +46,7 @@ TENxMatrix <- function(filepath, group="matrix")
 ### Taking advantage of sparsity
 ###
 
-setMethod("sparsity", "TENxMatrix", function(x) sparsity(x@seed))
+setMethod("nzcount", "TENxMatrix", function(x) nzcount(x@seed))
 
 setMethod("read_sparse_block", "TENxMatrix",
     function(x, viewport) read_sparse_block(x@seed, viewport)
@@ -56,12 +56,3 @@ setMethod("extractNonzeroDataByCol", "TENxMatrix",
     function(x, j) extractNonzeroDataByCol(x@seed, j)
 )
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Coercion to dgCMatrix
-###
-
-.from_TENxMatrix_to_dgCMatrix <- function(from) as(from@seed, "dgCMatrix")
-setAs("TENxMatrix", "dgCMatrix", .from_TENxMatrix_to_dgCMatrix)
-setAs("TENxMatrix", "sparseMatrix", .from_TENxMatrix_to_dgCMatrix)
-
