@@ -115,7 +115,7 @@ SEXP C_h5getdimscales(SEXP filepath, SEXP name, SEXP scalename)
 		H5Dclose(dset_id);
 		if (!isObject(filepath))
 			H5Fclose(file_id);
-		error(_HDF5Array_global_errmsg_buf());
+		error("%s", _HDF5Array_global_errmsg_buf());
 	}
 
 	ans = PROTECT(NEW_CHARACTER(h5dset.ndim));
@@ -129,7 +129,7 @@ SEXP C_h5getdimscales(SEXP filepath, SEXP name, SEXP scalename)
 			H5Dclose(dset_id);
 			if (!isObject(filepath))
 				H5Fclose(file_id);
-			error(_HDF5Array_global_errmsg_buf());
+			error("%s", _HDF5Array_global_errmsg_buf());
 		}
 		if (ret == 0) {
 			SET_STRING_ELT(ans, along, NA_STRING);
@@ -318,7 +318,7 @@ static SEXP check_scales(SEXP filepath, SEXP name, SEXP dimscales,
 		H5Dclose(dset_id);
 		if (!isObject(filepath))
 			H5Fclose(file_id);
-		error(_HDF5Array_global_errmsg_buf());
+		error("%s", _HDF5Array_global_errmsg_buf());
 	}
 
 	if (LENGTH(dimscales) > h5dset.ndim) {
@@ -366,7 +366,7 @@ static SEXP check_scales(SEXP filepath, SEXP name, SEXP dimscales,
 		H5Fclose(file_id);
 	UNPROTECT(1);
 	if (ret < 0)
-		error(_HDF5Array_global_errmsg_buf());
+		error("%s", _HDF5Array_global_errmsg_buf());
 	return ans;
 }
 
@@ -434,7 +434,7 @@ static void set_scales(SEXP filepath, SEXP name, SEXP dimscales,
 		H5Dclose(dset_id);
 		if (!isObject(filepath))
 			H5Fclose(file_id);
-		error(_HDF5Array_global_errmsg_buf());
+		error("%s", _HDF5Array_global_errmsg_buf());
 	}
 
 	for (along = 0; along < LENGTH(dimscales); along++) {
@@ -453,7 +453,7 @@ static void set_scales(SEXP filepath, SEXP name, SEXP dimscales,
 	if (!isObject(filepath))
 		H5Fclose(file_id);
 	if (ret < 0)
-		error(_HDF5Array_global_errmsg_buf());
+		error("%s", _HDF5Array_global_errmsg_buf());
 	return;
 }
 
@@ -526,7 +526,7 @@ SEXP C_h5getdimlabels(SEXP filepath, SEXP name)
 		H5Dclose(dset_id);
 		if (!isObject(filepath))
 			H5Fclose(file_id);
-		error(_HDF5Array_global_errmsg_buf());
+		error("%s", _HDF5Array_global_errmsg_buf());
 	}
 
 	/* First pass */
@@ -622,7 +622,7 @@ SEXP C_h5setdimlabels(SEXP filepath, SEXP name, SEXP dimlabels)
 				H5Fclose(file_id);
 			PRINT_TO_ERRMSG_BUF("H5DSset_label() failed on "
 					    "label %d", along + 1);
-			error(_HDF5Array_global_errmsg_buf());
+			error("%s", _HDF5Array_global_errmsg_buf());
 		}
 	}
 
