@@ -8,7 +8,7 @@
 /* A data structure for representing a viewport on a HDF5 dataset. */
 typedef struct {
 	hsize_t *h5off, *h5dim;
-	int *off, *dim; // same as h5off and h5dim but stored as int
+	size_t *off, *dim; // same as h5off and h5dim but stored as size_t
 } H5Viewport;
 
 #define	ALLOC_ALL_FIELDS		0
@@ -25,7 +25,7 @@ void _free_H5Viewport(H5Viewport *vp);
 
 hid_t _create_mem_space(
 	int ndim,
-	const int *dim
+	const size_t *dim
 );
 
 int _select_H5Viewport(
@@ -69,13 +69,12 @@ void _init_in_offset(
  * Inline helper functions
  */
 
-static inline int _next_midx(int ndim, const int *max_idx_plus_one,
-			     int *midx_buf)
+static inline int next_midx(int ndim,
+		const size_t *max_idx_plus_one, size_t *midx_buf)
 {
-	int along, i;
-
+	int along;
 	for (along = 0; along < ndim; along++) {
-		i = midx_buf[along] + 1;
+		size_t i = midx_buf[along] + 1;
 		if (i < max_idx_plus_one[along]) {
 			midx_buf[along] = i;
 			break;
@@ -85,37 +84,34 @@ static inline int _next_midx(int ndim, const int *max_idx_plus_one,
 	return along;
 }
 
-static inline void _update_in_offset(int ndim, SEXP index,
+static inline void update_in_offset(int ndim, SEXP index,
 		const hsize_t *h5chunkdim, const H5Viewport *mem_vp,
-		const int *inner_midx, int inner_moved_along,
+		const size_t *inner_midx, int inner_moved_along,
 		size_t *in_offset)
 {
-	SEXP start;
-	int i1, i0, along, h5along, di;
+	SEXP start = GET_LIST_ELT(index, inner_moved_along);
 	long long int in_off_inc;
-
-	start = GET_LIST_ELT(index, inner_moved_along);
 	if (start != R_NilValue) {
-		i1 = mem_vp->off[inner_moved_along] +
-		     inner_midx[inner_moved_along];
-		i0 = i1 - 1;
-		in_off_inc = _get_trusted_elt(start, i1) -
-			     _get_trusted_elt(start, i0);
+		R_xlen_t i1 = mem_vp->off[inner_moved_along] +
+			      inner_midx[inner_moved_along];
+		R_xlen_t i0 = i1 - 1;
+		in_off_inc = get_trusted_elt(start, i1) -
+			     get_trusted_elt(start, i0);
 	} else {
 		in_off_inc = 1;
 	}
 	if (inner_moved_along >= 1) {
-		along = inner_moved_along - 1;
-		h5along = ndim - inner_moved_along;
+		int along = inner_moved_along - 1;
+		int h5along = ndim - inner_moved_along;
 		do {
 			in_off_inc *= h5chunkdim[h5along];
-			di = 1 - mem_vp->dim[along];
+			R_xlen_t di = 1 - mem_vp->dim[along];
 			start = GET_LIST_ELT(index, along);
 			if (start != R_NilValue) {
-				i1 = mem_vp->off[along];
-				i0 = i1 - di;
-				in_off_inc += _get_trusted_elt(start, i1) -
-					      _get_trusted_elt(start, i0);
+				R_xlen_t i1 = mem_vp->off[along];
+				R_xlen_t i0 = i1 - di;
+				in_off_inc += get_trusted_elt(start, i1) -
+					      get_trusted_elt(start, i0);
 			} else {
 				in_off_inc += di;
 			}
