@@ -17,17 +17,14 @@
  */
 
 static size_t set_nchips(int ndim, SEXP starts,
-			  const int *ans_dim, int expand, int *nchips)
+			 const size_t *ans_dim, int expand, size_t *nchips)
 {
-	size_t total_num_chips;
-	int along, nchip;
-	SEXP start;
-
-	total_num_chips = 1;
-	for (along = 0; along < ndim; along++) {
-		start = GET_LIST_ELT(starts, along);
+	size_t total_num_chips = 1;
+	for (int along = 0; along < ndim; along++) {
+		SEXP start = GET_LIST_ELT(starts, along);
+		size_t nchip;
 		if (start != R_NilValue) {
-			nchip = LENGTH(start);
+			nchip = XLENGTH(start);
 		} else {
 			nchip = expand ? ans_dim[along] : 1;
 		}
@@ -72,61 +69,55 @@ static void init_h5dset_vp(const H5DSetDescriptor *h5dset,
 }
 
 static void update_h5dset_vp(int ndim,
-			const int *midx, int moved_along,
+			const size_t *midx, int moved_along,
 			SEXP starts, SEXP counts,
 			H5Viewport *h5dset_vp)
 {
-	int along, h5along, i;
-	SEXP start, count;
-
-	for (along = 0; along < ndim; along++) {
+	for (int along = 0; along < ndim; along++) {
 		if (along > moved_along)
 			break;
 		if (starts == R_NilValue)
 			continue;
-		start = VECTOR_ELT(starts, along);
+		SEXP start = VECTOR_ELT(starts, along);
 		if (start == R_NilValue)
 			continue;
-		h5along = ndim - 1 - along;
-		i = midx[along];
-		h5dset_vp->h5off[h5along] = _get_trusted_elt(start, i) - 1;
+		int h5along = ndim - 1 - along;
+		size_t i = midx[along];
+		h5dset_vp->h5off[h5along] = get_trusted_elt(start, i) - 1;
 		if (counts == R_NilValue)
 			continue;
-		count = VECTOR_ELT(counts, along);
+		SEXP count = VECTOR_ELT(counts, along);
 		if (count == R_NilValue)
 			continue;
-		h5dset_vp->h5dim[h5along] = _get_trusted_elt(count, i);
+		h5dset_vp->h5dim[h5along] = get_trusted_elt(count, i);
 	}
 	return;
 }
 
-/* Return nb of hyperslabs (or -1 on error). */
+/* Returns nb of hyperslabs (or -1 on error). */
 static long long int select_hyperslabs(const H5DSetDescriptor *h5dset,
-			SEXP starts, SEXP counts, const int *ans_dim,
-			int *nchips, int *midx_buf)
+			SEXP starts, SEXP counts, const size_t *ans_dim,
+			size_t *nchips_buf, size_t *midx_buf)
 {
-	int ret, ndim, moved_along;
-	H5Viewport h5dset_vp;
-	long long int num_hyperslabs;
-
-	ret = H5Sselect_none(h5dset->h5space_id);
+	int ret = H5Sselect_none(h5dset->h5space_id);
 	if (ret < 0) {
 		PRINT_TO_ERRMSG_BUF("H5Sselect_none() returned an error");
 		return -1;
 	}
 
-	ndim = h5dset->ndim;
-	set_nchips(ndim, starts, ans_dim, 0, nchips);
+	int ndim = h5dset->ndim;
+	set_nchips(ndim, starts, ans_dim, 0, nchips_buf);
 
 	/* Allocate 'h5dset_vp'. */
+	H5Viewport h5dset_vp;
 	if (_alloc_H5Viewport(&h5dset_vp, ndim, ALLOC_H5OFF_AND_H5DIM) < 0)
 		return -1;
 
 	init_h5dset_vp(h5dset, starts, &h5dset_vp);
 
 	/* Walk on the hyperslabs. */
-	num_hyperslabs = 0;
-	moved_along = ndim;
+	long long int num_hyperslabs = 0;
+	int moved_along = ndim;
 	do {
 		num_hyperslabs++;
 		update_h5dset_vp(ndim, midx_buf, moved_along,
@@ -136,7 +127,7 @@ static long long int select_hyperslabs(const H5DSetDescriptor *h5dset,
 						     &h5dset_vp);
 		if (ret < 0)
 			break;
-		moved_along = _next_midx(ndim, nchips, midx_buf);
+		moved_along = next_midx(ndim, nchips_buf, midx_buf);
 	} while (moved_along < ndim);
 	//printf("nb of hyperslabs = %lld\n", num_hyperslabs);
 
@@ -144,53 +135,47 @@ static long long int select_hyperslabs(const H5DSetDescriptor *h5dset,
 	return ret < 0 ? -1 : num_hyperslabs;
 }
 
-static inline hsize_t *add_element(int ndim, const int *midx,
+static inline hsize_t *add_element(int ndim, const size_t *midx,
 				   SEXP starts, hsize_t *coord_p)
 {
-	int h5along, i;
-	SEXP start;
-	long long int coord;
-
-	for (h5along = ndim - 1; h5along >= 0; h5along--) {
-		i = midx[h5along];
-		start = GET_LIST_ELT(starts, h5along);
+	for (int h5along = ndim - 1; h5along >= 0; h5along--) {
+		size_t i = midx[h5along];
+		SEXP start = GET_LIST_ELT(starts, h5along);
+		hsize_t coord;
 		if (start != R_NilValue) {
-			coord = _get_trusted_elt(start, i) - 1;
+			coord = get_trusted_elt(start, (R_xlen_t) i) - 1;
 		} else {
-			coord = i;
+			coord = (hsize_t) i;
 		}
-		*(coord_p++) = (hsize_t) coord;
+		*(coord_p++) = coord;
 	}
 	return coord_p;
 }
 
-/* Return nb of selected elements (or -1 on error). */
+/* Returns nb of selected elements (or -1 on error). */
 static long long int select_elements(const H5DSetDescriptor *h5dset,
-			SEXP starts, const int *ans_dim,
-			int *nchips, int *midx_buf)
+			SEXP starts, const size_t *ans_dim,
+			size_t *nchips_buf, size_t *midx_buf)
 {
-	int ndim, outer_moved_along, ret;
-	size_t num_elements;
-	hsize_t *coord_buf, *coord_p;
-
-	ndim = h5dset->ndim;
-	num_elements = set_nchips(ndim, starts, ans_dim, 1, nchips);
+	int ndim = h5dset->ndim;
+	size_t num_elements = set_nchips(ndim, starts, ans_dim, 1, nchips_buf);
 
 	/* Allocate 'coord_buf'. */
-	coord_buf = _alloc_hsize_t_buf(num_elements * ndim, 0, "'coord_buf'");
+	hsize_t *coord_buf = _alloc_hsize_t_buf(num_elements * ndim,
+						0, "'coord_buf'");
 	if (coord_buf == NULL)
 		return -1;
 
 	/* Walk on the selected elements. */
-	coord_p = coord_buf;
-	outer_moved_along = ndim;
+	hsize_t *coord_p = coord_buf;
+	int outer_moved_along = ndim;
 	do {
 		coord_p = add_element(ndim, midx_buf, starts, coord_p);
-		outer_moved_along = _next_midx(ndim, nchips, midx_buf);
+		outer_moved_along = next_midx(ndim, nchips_buf, midx_buf);
 	} while (outer_moved_along < ndim);
 
-	ret = H5Sselect_elements(h5dset->h5space_id, H5S_SELECT_APPEND,
-				 num_elements, coord_buf);
+	int ret = H5Sselect_elements(h5dset->h5space_id, H5S_SELECT_APPEND,
+				     num_elements, coord_buf);
 	free(coord_buf);
 	if (ret < 0)
 		return -1;
@@ -198,21 +183,18 @@ static long long int select_elements(const H5DSetDescriptor *h5dset,
 }
 
 static int set_h5selection(const H5DSetDescriptor *h5dset, int method,
-			   SEXP starts, SEXP counts, const int *ans_dim)
+			   SEXP starts, SEXP counts, const size_t *ans_dim)
 {
-	int ndim;
-	IntAE *nchip_buf, *midx_buf;
+	int ndim = h5dset->ndim;
+	size_t *nchip_buf = R_alloc0_size_t_array(ndim);
+	size_t *midx_buf = R_alloc0_size_t_array(ndim);
+
 	long long int total_num_chips;
-
-	ndim = h5dset->ndim;
-	nchip_buf = new_IntAE(ndim, ndim, 0);
-	midx_buf = new_IntAE(ndim, ndim, 0);
-
 	//clock_t t0 = clock();
 	if (method == 1) {
 		total_num_chips = select_hyperslabs(h5dset,
 					starts, counts, ans_dim,
-					nchip_buf->elts, midx_buf->elts);
+					nchip_buf, midx_buf);
 	} else {
 		if (counts != R_NilValue) {
 			PRINT_TO_ERRMSG_BUF("'counts' must be NULL when "
@@ -221,7 +203,7 @@ static int set_h5selection(const H5DSetDescriptor *h5dset, int method,
 		}
 		total_num_chips = select_elements(h5dset,
 					starts, ans_dim,
-					nchip_buf->elts, midx_buf->elts);
+					nchip_buf, midx_buf);
 	}
 	//double dt = (1.0 * clock() - t0) / CLOCKS_PER_SEC;
 	//printf("time for setting h5 selection: %e\n", dt);
@@ -231,7 +213,7 @@ static int set_h5selection(const H5DSetDescriptor *h5dset, int method,
 }
 
 static int read_data_1_2(const H5DSetDescriptor *h5dset, int method,
-		SEXP starts, SEXP counts, const int *ans_dim,
+		SEXP starts, SEXP counts, const size_t *ans_dim,
 		hid_t mem_space_id, void *mem)
 {
 	int ret;
@@ -258,26 +240,21 @@ static int read_data_1_2(const H5DSetDescriptor *h5dset, int method,
 
 static int read_hyperslab(const H5DSetDescriptor *h5dset,
 		SEXP starts, SEXP counts,
-		const int *midx, int moved_along,
+		const size_t *midx, int moved_along,
 		H5Viewport *h5dset_vp,
 		hid_t mem_space_id, void *mem,
 		H5Viewport *mem_vp)
 {
-	int ndim, along, h5along, i;
-	SEXP start;
-
-	ndim = h5dset->ndim;
-
+	int ndim = h5dset->ndim;
 	/* Update 'mem_vp' and 'h5dset_vp' IN THAT ORDER! */
-	for (along = 0; along < ndim; along++) {
+	for (int along = 0; along < ndim; along++) {
 		if (along > moved_along)
 			break;
-		start = GET_LIST_ELT(starts, along);
+		SEXP start = GET_LIST_ELT(starts, along);
 		if (start == R_NilValue)
 			continue;
-		h5along = ndim - 1 - along;
-		i = midx[along];
-		if (i == 0) {
+		int h5along = ndim - 1 - along;
+		if (midx[along] == 0) {
 			mem_vp->h5off[h5along] = 0;
 		} else {
 			mem_vp->h5off[h5along] += h5dset_vp->h5dim[h5along];
@@ -291,23 +268,20 @@ static int read_hyperslab(const H5DSetDescriptor *h5dset,
 }
 
 static int read_data_3(const H5DSetDescriptor *h5dset,
-		SEXP starts, SEXP counts, const int *ans_dim,
+		SEXP starts, SEXP counts, const size_t *ans_dim,
 		hid_t mem_space_id, void *mem)
 {
-	int ndim, moved_along, ret;
-	H5Viewport h5dset_vp, mem_vp;
-	IntAE *nchip_buf, *midx_buf;
-	long long int num_hyperslabs;
+	int ndim = h5dset->ndim;
+	size_t *nchip_buf = R_alloc0_size_t_array(ndim);
+	size_t *midx_buf = R_alloc0_size_t_array(ndim);
 
-	ndim = h5dset->ndim;
-	nchip_buf = new_IntAE(ndim, ndim, 0);
-	midx_buf = new_IntAE(ndim, ndim, 0);
-
-	set_nchips(ndim, starts, ans_dim, 0, nchip_buf->elts);
+	set_nchips(ndim, starts, ans_dim, 0, nchip_buf);
 
 	/* Allocate 'h5dset_vp' and 'mem_vp'. */
+	H5Viewport h5dset_vp;
 	if (_alloc_H5Viewport(&h5dset_vp, ndim, ALLOC_H5OFF_AND_H5DIM) < 0)
 		return -1;
+	H5Viewport mem_vp;
 	mem_vp.h5off = _alloc_hsize_t_buf(ndim, 1, "'mem_vp.h5off'");
 	if (mem_vp.h5off == NULL) {
 		_free_H5Viewport(&h5dset_vp);
@@ -319,19 +293,18 @@ static int read_data_3(const H5DSetDescriptor *h5dset,
 	init_h5dset_vp(h5dset, starts, &h5dset_vp);
 
 	/* Walk on the hyperslabs. */
-	num_hyperslabs = 0;
-	moved_along = ndim;
+	long long int num_hyperslabs = 0;
+	int moved_along = ndim, ret;
 	do {
 		num_hyperslabs++;
 		ret = read_hyperslab(h5dset, starts, counts,
-				     midx_buf->elts, moved_along,
+				     midx_buf, moved_along,
 				     &h5dset_vp,
 				     mem_space_id, mem,
 				     &mem_vp);
 		if (ret < 0)
 			break;
-		moved_along = _next_midx(ndim, nchip_buf->elts,
-					 midx_buf->elts);
+		moved_along = next_midx(ndim, nchip_buf, midx_buf);
 	} while (moved_along < ndim);
 
 	//printf("nb of hyperslabs = %lld\n", num_hyperslabs);
@@ -347,99 +320,99 @@ static int read_data_3(const H5DSetDescriptor *h5dset,
 
 static long long int check_uaselection_against_h5dset(
 		const H5DSetDescriptor *h5dset,
-		SEXP starts, SEXP counts, int *uaselection_dim_buf)
+		SEXP starts, SEXP counts, size_t *ans_dim)
 {
-	int ndim, along, h5along;
-	LLongAE *dim_buf;
-
-	ndim = h5dset->ndim;
-	dim_buf = new_LLongAE(ndim, ndim, 0);
+	int ndim = h5dset->ndim;
+	int along, h5along;
 	for (along = 0, h5along = ndim - 1; along < ndim; along++, h5along--)
-		dim_buf->elts[along] =
-			(long long int) h5dset->h5dim[h5along];
-	return _check_uaselection(ndim, dim_buf->elts, starts, counts,
-				  uaselection_dim_buf);
+		ans_dim[along] = (size_t) h5dset->h5dim[h5along];
+	return _check_uaselection(ndim, ans_dim, starts, counts, ans_dim);
 }
 
 static long long int check_ordered_uaselection_against_h5dset(
 		const H5DSetDescriptor *h5dset,
-		SEXP starts, SEXP counts, int *uaselection_dim_buf,
-		int *nstart_buf, int *nchip_buf,
-		long long int *last_chip_start_buf)
+		SEXP starts, SEXP counts, size_t *ans_dim,
+		size_t *nstart_buf, size_t *nchip_buf,
+		size_t *last_chip_start_buf)
 {
-	int ndim, along, h5along;
-	LLongAE *dim_buf;
-
-	ndim = h5dset->ndim;
-	dim_buf = new_LLongAE(ndim, ndim, 0);
+	int ndim = h5dset->ndim;
+	int along, h5along;
 	for (along = 0, h5along = ndim - 1; along < ndim; along++, h5along--)
-		dim_buf->elts[along] =
-			(long long int) h5dset->h5dim[h5along];
-	return _check_ordered_uaselection(ndim, dim_buf->elts, starts, counts,
-					  uaselection_dim_buf,
-					  nstart_buf, nchip_buf,
+		ans_dim[along] = (size_t) h5dset->h5dim[h5along];
+	return _check_ordered_uaselection(ndim, ans_dim, starts, counts,
+					  ans_dim, nstart_buf, nchip_buf,
 					  last_chip_start_buf);
 }
 
-/* Implements methods 1 to 3.
-   Return an ordinary array or R_NilValue if an error occured. */
-SEXP _h5mread_startscounts(const H5DSetDescriptor *h5dset,
-			   SEXP starts, SEXP counts, int noreduce,
-			   int method, int *ans_dim)
+/* Returns 'list(starts, counts)' or R_NilValue if an error occured. */
+SEXP _compute_startscounts_ans_dim(const H5DSetDescriptor *h5dset,
+		SEXP starts, SEXP counts, int noreduce, int method,
+		size_t *ans_dim)
 {
-	int ndim, ret;
-	long long int ans_len;
-	IntAE *nstart_buf, *nchip_buf;
-	LLongAE *last_chip_start_buf;
-	SEXP ans, reduced;
-	void *mem;
-	hid_t mem_space_id;
-	int nprotect = 0;
+	SEXP startscounts = PROTECT(NEW_LIST(2));
+	SET_VECTOR_ELT(startscounts, 0, starts);
+	SET_VECTOR_ELT(startscounts, 1, counts);
 
-	ndim = h5dset->ndim;
+	long long int ans_len;
 	if (noreduce || method == 2) {
 		/* This call will populate 'ans_dim'. */
 		ans_len = check_uaselection_against_h5dset(h5dset,
-					starts, counts, ans_dim);
+						    starts, counts, ans_dim);
+		UNPROTECT(1);
 		if (ans_len < 0)
 			return R_NilValue;
-	} else {
-		nstart_buf = new_IntAE(ndim, ndim, 0);
-		nchip_buf = new_IntAE(ndim, ndim, 0);
-		last_chip_start_buf = new_LLongAE(ndim, ndim, 0);
-		/* This call will populate 'ans_dim', 'nchip_buf',
-		   and 'last_chip_start_buf'. */
-		ans_len = check_ordered_uaselection_against_h5dset(h5dset,
-					starts, counts, ans_dim,
-					nstart_buf->elts, nchip_buf->elts,
-					last_chip_start_buf->elts);
-		if (ans_len < 0)
-			return R_NilValue;
-		if (_uaselection_can_be_reduced(ndim, nstart_buf->elts,
-						nchip_buf->elts))
-		{
-			reduced = PROTECT(_reduce_uaselection(
-						ndim, starts, counts, ans_dim,
-						nchip_buf->elts,
-						last_chip_start_buf->elts));
-			nprotect++;
-			starts = VECTOR_ELT(reduced, 0);
-			counts = VECTOR_ELT(reduced, 1);
-		}
+		return startscounts;
 	}
 
-	ans = PROTECT(allocVector(h5dset->h5type->Rtype, (R_xlen_t) ans_len));
-	nprotect++;
+	int ndim = h5dset->ndim;
+	size_t *nstart_buf = R_alloc0_size_t_array(ndim);
+	size_t *nchip_buf  = R_alloc0_size_t_array(ndim);
+	size_t *last_chip_start_buf = R_alloc0_size_t_array(ndim);
+	/* This call will populate 'ans_dim', 'nstart_buf', 'nchip_buf',
+	   and 'last_chip_start_buf'. */
+	ans_len = check_ordered_uaselection_against_h5dset(h5dset,
+					starts, counts, ans_dim,
+					nstart_buf, nchip_buf,
+					last_chip_start_buf);
+	if (ans_len < 0) {
+		UNPROTECT(1);
+		return R_NilValue;
+	}
+	if (!_uaselection_can_be_reduced(ndim, starts, nstart_buf, nchip_buf)) {
+		UNPROTECT(1);
+		return startscounts;
+	}
+	SEXP reduced = _reduce_uaselection(ndim, starts, counts, ans_dim,
+					   nchip_buf,
+					   last_chip_start_buf);
+	UNPROTECT(1);
+	return reduced;
+}
+
+/* Implements methods 1 to 3.
+   Returns an ordinary array or R_NilValue if an error occured. */
+SEXP _h5mread_startscounts(const H5DSetDescriptor *h5dset,
+		SEXP startscounts, int noreduce, int method,
+		const size_t *ans_dim)
+{
+	int ndim = h5dset->ndim;
+	R_xlen_t ans_len = 1;
+	for (int along = 0; along < ndim; along++)
+		ans_len *= ans_dim[along];
+	SEXP ans = PROTECT(allocVector(h5dset->h5type->Rtype, ans_len));
 
 	if (ans_len != 0) {
-		mem = DATAPTR(ans);
+		void *mem = DATAPTR(ans);
 		if (mem == NULL)
 			goto on_error;
 
-		mem_space_id = _create_mem_space(ndim, ans_dim);
+		hid_t mem_space_id = _create_mem_space(ndim, ans_dim);
 		if (mem_space_id < 0)
 			goto on_error;
 
+		SEXP starts = VECTOR_ELT(startscounts, 0);
+		SEXP counts = VECTOR_ELT(startscounts, 1);
+		int ret;
 		if (method <= 2) {
 			ret = read_data_1_2(h5dset, method,
 					starts, counts, ans_dim,
@@ -454,11 +427,11 @@ SEXP _h5mread_startscounts(const H5DSetDescriptor *h5dset,
 			goto on_error;
 	}
 
-	UNPROTECT(nprotect);
+	UNPROTECT(1);
 	return ans;
 
     on_error:
-	UNPROTECT(nprotect);
+	UNPROTECT(1);
 	return R_NilValue;
 }
 
