@@ -4,11 +4,21 @@
 #include "H5DSetDescriptor.h"
 #include <Rdefines.h>
 
+SEXP _compute_startscounts_ans_dim(
+	const H5DSetDescriptor *h5dset,
+	SEXP starts,
+	SEXP counts,
+	int noreduce,
+	int method,
+	size_t *ans_dim
+);
+
 SEXP _h5mread_startscounts(
 	const H5DSetDescriptor *h5dset,
-	SEXP starts, SEXP counts, int noreduce,
+	SEXP startscounts,
+	int noreduce,
 	int method,
-	int *ans_dim
+	const size_t *ans_dim
 );
 
 #endif  /* _H5MREAD_STARTSCOUNTS_H_ */
